'' Engine for Saimazoom-like games.
'' Designed to be partially reusable.

'' Constants

' Main states
Const STATESTILL as uByte = 0
Const STATEWALKING as uByte = 1
Const STATEIDLE as uByte = 2
Const STATESPAWNING as uByte = 3
Const STATEDYING as uByte = 4
Const STATECOUNTDOWN as uByte = 5

' For timed states (in numer of game frames)
Const TIMESPAWNING as uByte = 12
Const TIMEDYING as uByte = 8

' Directions
Const DIRUP as uByte = 1
Const DIRDOWN as uByte = 0
Const DIRLEFT as uByte = 3
Const DIRRIGHT as uByte = 2

' Map dimmensions
Const MAPWIDTH as uByte = 7
Const MAPHEIGHT as uByte = 7
Const MAPLASTSCREEN as uByte = 48	' Last screen = MAPWIDTH * MAPHEIGHT - 1

' Screen dimmensions
Const SCREENWIDTH as uByte = 9
Const SCREENHEIGHT as uByte = 7
Const SCREENMAX as uByte = 62
Const SCREENSIZE as uByte = 63
Const SCREENX as uByte = 1
Const SCREENY as uByte = 1

' Objects
Const FIRSTOBJECT as uByte = 1
Const LASTOBJECT as uByte = 8

' Enemy (re)spawn ratio. The bigger, the less
Const ENEMYRESPAWNRATIOAFTERKILLED as uByte = 128
Const ENEMYRESPAWNRATIOWHENFRESH as uByte = 32

' Tile types
Const TILENONWALKABLE as uByte = 8
Const TILEOBJECT as uByte = 9
Const TILEBREAKABLE as uByte = 10

' To restore the map
Const MAXRESTORABLETILES as uByte = 20
Const MAXBOMBS as uByte = 10

'' Variables

Dim nPant as uByte

Dim plX as Byte
Dim plY as Byte
Dim plMX as Byte
Dim plMY as Byte
Dim plNXr as Byte
Dim plNYr as Byte
Dim plXr as Byte
Dim plYr as Byte
Dim plState as uByte
Dim plCt as uByte
Dim plFrame as uByte
Dim plScore as uInteger
Dim plLives as uByte
Dim plFacing as uByte
Dim plBombs as uByte

Dim enX (1) as Byte
Dim enY (1) as Byte
Dim enMX (1) as Byte
Dim enMY (1) as Byte
Dim enXr (1) as Byte
Dim enYr (1) as Byte
Dim enState (1) as uByte
Dim enCt (1) as uByte
Dim enFrame (1) as uByte
Dim enWasKilled (1) as uByte
Dim enType (1) as uByte
Dim enSubCt (1) as uByte
Dim enFacing (1) as uByte
Dim enUpdateMe (1) as uByte

Dim bombX as Byte
Dim bombY as Byte
Dim bombTimer as Byte
Dim bombState as uByte
Dim bombCt as Byte

Dim bombScare as Byte

Dim objContainers (MAPLASTSCREEN) as uByte

Dim restorableTilesIndex (MAXRESTORABLETILES) as uInteger
Dim restorableTilesValue (MAXRESTORABLETILES) as uByte
Dim restorableIndex as uByte

'' Subs and functions

' Restorable tiles: Some modifications are done to the map.
' We track them in these arrays so we can restore the map
' next time we play.

' Call this at the beginning of your program:
Sub initRestorableTiles ()
	For i = 0 To MAXRESTORABLETILES
		restorableTilesIndex (i) = 9999
	Next i
End Sub

' Add new
Sub addRestorableTile(index as uInteger, value as uByte)
	restorableTilesIndex (restorableIndex) = index
	restorableTilesValue (restorableIndex) = value
	restorableIndex = restorableIndex + 1
End Sub

' Restore. Call this when initing a new game
Sub restoreRestorableTiles ()
	Dim i as uByte
	For i = 0 To MAXRESTORABLETILES
		If (restorableTilesIndex (i) <> 9999) Then
			mapa (restorableTilesIndex(i)) = restorableTilesValue (i)
			restorableTilesIndex (i) = 9999
		End If
	Next i
	restorableIndex = 0
End Sub

' This function initializes all object containers.
' 0 means this container doesn't exist.
Sub initObjContainers ()
	Dim i as uByte
	Dim j as uByte
	Dim idx as uInteger
	For i = 0 To MAPLASTSCREEN
		objContainers (i) = 0
		' Find object in map
		idx = Cast (Integer, i) * SCREENSIZE
		For j = 0 TO SCREENMAX
			If mapa (idx + j) >= FIRSTOBJECT And mapa (idx + j) <= LASTOBJECT Then
				objContainers (i) = mapa (idx + j)
			End If
		Next j
	Next i
End Sub

' Spawns some bombs accross the map. Saves the original tiles.
Sub initBombs ()
	Dim i as uByte
	Dim randPant as uByte
	Dim rX as uByte
	Dim rY as uByte
	Dim shuffle as uInteger
	Dim mapSize as uInteger
	
	mapSize = MAPWIDTH * MAPHEIGHT
	
	For i = 1 To MAXBOMBS
		Do
			randPant = Int (Rnd * mapSize)
			rX = Int (Rnd * SCREENWIDTH - 2) + 1
			rY = Int (Rnd * SCREENHEIGHT- 2) + 1
			
			shuffle = Cast (Integer,randPant) * SCREENSIZE + rX + SCREENWIDTH * rY	
		Loop While behaviour (mapa (shuffle)) >= TILENONWALKABLE Or mapa (shuffle) = 9
		addRestorableTile (shuffle, mapa (shuffle))
		mapa (shuffle) = 9 ' this is a bomb
	Next i
End Sub

' This function inits the game. Sets up everything
' needed. Checks the "restore" variable to see if
' there's something to be restored and restores it.
Sub initGame ()
	' Restore map
	restoreRestorableTiles ()

	' Player values

	nPant = 17
	plXr = 5
	plYr = 3
	plX = 3 * plXr
	plY = 3 * plYr
	plState = STATESTILL
	plCt = 0
	plFrame = 0
	plFacing = DIRDOWN
	plLives = 10
	plScore = 0
	plBombs = 3 
End Sub

Sub getSprite0NewBackground ()
	' Get sprite 0 new background.
	Asm
		ld	a, (sbc_y0)
		ld	b, a
		ld	a, (sbc_x0)
		ld	c, a
		ld	de, subaco_sprite_buffer_0
		call subaco_copy_to_buffer
	End Asm
End Sub

' Restorable objects

' Tile addresses to map attributes. Uses global
' variable nPant.
Function getMapIndex (tileX as uByte, tileY as uByte) as uInteger
	' Return nPant * SCREENSIZE + tileY * SCREENWIDTH + tileX
	' Each screen has 7*9 = 63 bytes. 63 * nPant = 64*nPant-nPant = (nPant<<6) - nPant
	' 9*tileY = 8*tileY + tileY = (tileY<<3) + tileY
	Return (Cast (Integer, nPant) << 6) - nPant + (tileY << 3) + tileY + tileX
End Function

' These functios detect some kind of tiles
Function isWalkable (tileX as uByte, tileY as uByte) as uByte
	Return (behaviour (mapa (getMapIndex (tileX, tileY))) < TILENONWALKABLE)
End Function

Function isObjectContainer (tileX as uByte, tileY as uByte) as uByte
	Return (behaviour (mapa (getMapIndex (tileX, tileY))) = TILEOBJECT)
End Function

Function isObjectBreakable (tileX as uByte, tileY as uByte) as uByte
	Return (behaviour (mapa (getMapIndex (tileX, tileY))) = TILEBREAKABLE)
End Function

' These function do special actions with some kinds of tiles
Sub doObjectContainer (tileX as uByte, tileY as uByte)
	Dim itemOut as uByte
	Dim itemIn as uByte
	itemIn = objContainers (nPant)
	itemOut = inventoryShuffle (itemIn)
	objContainers (nPant) = itemOut
	drawInventory ()
	subacoDrawTile (SCREENX + 3 * tileX, SCREENY + 3 * tileY, itemOut)
	makeSound (SOUNDTOGGLE)
End Sub

Sub doObjectBreakable (tileX as uByte, tileY as uByte)
	' I have to remove the object (with PUF animation)
	' And then add it to the list of restorable objects
	' So next time we play, the object is restored.
	Dim tileOut as uByte
	Dim postn as uByte
	Dim breakIt as uByte
	breakIt = 0
	tileOut = mapa (getMapIndex (tileX, tileY))
	'' CUSTOM CODE: YOU'LL HAVE TO CHANGE THIS
	If tileOut = 13 Then 
		If inventoryIfExistsPickOut (1) Then
			breakIt = 1
		End If
	ElseIf tileOut = 14 Then
		If inventoryIfExistsPickOut (2) Then
			breakIt = 1
		End If
	ElseIf tileOut = 15 Then
		If inventoryIfExistsPickOut (3) Then
			breakIt = 1
		End If
	End If
	'' END OF CUSTOM CODE
	If breakIt = 1 Then
		addRestorableTile (getMapIndex (tileX, tileY), tileOut)
		'' CUSTOM CODE: WHICH TILE TO PUT WHEN BREAK OBJECT
		mapa (getMapIndex (tileX, tileY)) = 74
		Asm
			call subaco_restore_background
		End Asm
		subacoDrawTile (SCREENX + 3 * tileX, SCREENY + 3 * tileY, 74)
		Asm
			call subaco_fill_buffers
			call subaco_draw_sprites
		End Asm
		'' END OF CUSTOM CODE
		makeSound (SOUNDBREAK)
		drawInventory ()
	End If
End Sub

' This function reads the keyboard to update the
' player position.
Sub movePlayer ()
	If plState = STATESTILL Then
		plMX = 0: plMY = 0
		
		'' Get bomb?
		If mapa (getMapIndex (plXr, plYr)) = 9 Then
			mapa (getMapIndex (plXr, plYr)) = 74
			Asm
				call subaco_restore_background
			End Asm
			subacoDrawTile (SCREENX + 3 * plXr, SCREENY + 3 * plYr, 74)
			Asm
				call subaco_fill_buffers
				call subaco_draw_sprites
			End Asm
			'getSprite0NewBackground ()
			'subacoMoveSprites ()
			makeSound (SOUNDSETBOMB)
			plBombs = plBombs + 1
			showBombs ()
		End If
		
		'' Drop bomb?
		If (In (32766) bAnd 1) = 0 Or (In (32766) bAnd 4) = 0 Then
			fireBomb ()
      	End if
		
		'' Change screen?
		changeScreen ()
		
		'' Detectar "O": Sexta semifila, bit 1
		If (In (57342) bAnd 2) = 0 And plXr > 0 Then 
			plFacing = DIRLEFT
			If isWalkable (plXr - 1, plYr) Then
				plState = STATEWALKING
				plMX = -1
				plNXr = plXr - 1
			ElseIf isObjectContainer (plXr - 1, plYr) Then
				doObjectContainer (plXr - 1, plYr)
			ElseIf isObjectBreakable (plXr - 1, plYr) Then
				doObjectBreakable (plXr - 1, plYr)
			End If
		'' Detectar "P": Sexta semifila, bit 0
		ElseIf (In (57342) bAnd 1) = 0 And plXr < SCREENWIDTH - 1 Then 
			plFacing = DIRRIGHT
			If isWalkable (plXr + 1, plYr) Then
				plState = STATEWALKING
				plMX = 1
				plNXr = plXr + 1
			ElseIf isObjectContainer (plXr + 1, plYr) Then
				doObjectContainer (plXr + 1, plYr)
			ElseIf isObjectBreakable (plXr + 1, plYr) Then
				doObjectBreakable (plXr + 1, plYr)
			End If
		End If
		
		'' Detectar "Q": Tercera semifila, bit 0
		If (In (64510) bAnd 1) = 0 And plYr > 0 Then
			plFacing = DIRUP
			If isWalkable (plXr, plYr - 1) And isWalkable (plNXr, plYr - 1) Then
				plState = STATEWALKING
				plMY = -1
				plNYr = plYr - 1
			ElseIf isObjectContainer (plXr, plYr - 1) Then
				doObjectContainer (plXr, plYr - 1)
			ElseIf isObjectBreakable (plXr, plYr - 1) Then
				doObjectBreakable (plXr, plYr - 1)
			End If
		'' Detectar "A": Segunda semifila, bit 0
		ElseIf (In (65022) bAnd 1) = 0  And plYr < SCREENHEIGHT - 1 Then
			plFacing = DIRDOWN
			If isWalkable (plXr, plYr + 1) And isWalkable (plNXr, plYr + 1) Then
				plState = STATEWALKING
				plMY = 1
				plNYr = plYr + 1
			ElseIf isObjectContainer (plXr, plYr + 1) Then
				doObjectContainer (plXr, plYr + 1)
			ElseIf isObjectBreakable (plXr, plYr + 1) Then
				doObjectBreakable (plXr, plYr + 1)
			End If
		End If
		
	End If
	
	If plState = STATEWALKING Then
		'' Move right away
		plX = plX + plMX
		plY = plY + plMY
		makeSound (SOUNDSTEP)
		plFrame = 1 - plFrame
		plCt = plCt + 1
		If plCt = 3 Then
			plCt = 0
			plState = STATESTILL
			plXr = plNXr
			plYr = plNYr
		End If
	End If
End Sub

' This function applies uber incredible stupid AI
' to move the enemy.
Sub moveEnemies ()
	Dim shuffled as uByte
	Dim i as uByte

	For i = 0 To 1
		enUpdateMe (i) = 1 - enUpdateMe (i)
		If enState (i) = STATEIDLE Then
			' Spawn a new enemy. If an enemy was already killed,
			' it takes harder to respawn.
			If enWasKilled (i) Then
				shuffled = Int (Rnd * ENEMYRESPAWNRATIOAFTERKILLED)
			Else
				shuffled = Int (Rnd * ENEMYRESPAWNRATIOWHENFRESH)
			End If	
			If shuffled = 1 And bombScare = 0 Then
				' Search for a walkable tile
				Do
					enXr (i) = Int (Rnd * 9)
					enYr (i) = Int (Rnd * 7)
				Loop While Not isWalkable (enXr (i), enYr (i)) Or (enXr (i) = plX And enYr (i) = plY)
				' Enemy type (Don't repeat!!)
				Do
					enType (i) = Int (Rnd * 4)
				Loop While enType (i) = enType (1 - i)
				' Coordinates
				enX (i) = enXr (i) * 3
				enY (i) = enYr (i) * 3
				enWasKilled (i) = 0
				enCt (i) = 0
				enFrame (i) = 0
				' Spawn new enemy
				enState (i) = STATESPAWNING
				subacoActivateSprite (1 + i)
			End If
		ElseIf enState (i) = STATESPAWNING Then
			enCt (i) = enCt (i) + 1
			If enCt (i) = TIMESPAWNING Then
				enCt (i) = 0
				enState (i) = STATESTILL
			End If
		ElseIf enState (i) = STATESTILL Then
			enFrame (i) = 1 - enFrame (i)
			shuffled = Int (Rnd * 8)
			If shuffled < 4 Then
				enState (i) = STATEWALKING
				enCt (i) = 1 + Int (Rnd * 3)
				enSubCt (i) = 0
				enFacing (i) = shuffled
			End If
		ElseIf enState (i) = STATEWALKING Then
			If enUpdateMe (i) Then
				enFrame (i) = 1 - enFrame (i)
				If enSubCt (i) = 0 Then
					enMX (i) = 0: enMY (i) = 0
					' Check if we can move in the prefered direction
					If enFacing (i) = DIRLEFT And enXr (i) > 0 Then
						If isWalkable (enXr (i) - 1, enYr (i)) Then
							enMX (i) = -1
							enXr (i) = enXr (i) - 1
						End If
					ElseIf enFacing (i) = DIRRIGHT And enXr (i) < SCREENWIDTH - 1 Then
						If isWalkable (enXr (i) + 1, enYr (i)) Then
							enMX (i) = 1
							enXr (i) = enXr (i) + 1
						End If
					ElseIf enFacing (i) = DIRUP And enYr (i) > 0 Then
						If isWalkable (enXr (i), enYr (i) - 1) Then
							enMY (i) = -1
							enYr (i) = enYr (i) - 1
						End If
					ElseIf enFacing (i) = DIRDOWN And enYr (i) < SCREENHEIGHT - 1 Then
						If isWalkable (enXr (i), enYr (i) + 1) Then
							enMY (i) = 1
							enYr (i) = enYr (i) + 1
						End If
					End If
				End if
				
				enX (i) = enX (i) + enMX (i)
				enY (i) = enY (i) + enMY (i)
				
				enSubCt (i) = enSubCt (i) + 1
				If enSubCt (i) = 3 Then 
					enSubCt (i) = 0
					enCt (i) = enCt (i) - 1
					If enCt (i) = 0 Then
						enState (i) = STATESTILL
					End If
				End If
			End If
		ElseIf enState (i) = STATEDYING Then
			enCt (i) = enCt (i) + 1
			If enCt (i) = TIMEDYING Then
				enCt (i) = 0
				enState (i) = STATEIDLE
				subacoDeactivateSprite (1 + i)
				enWasKilled (i) = 1
			End If
		End If
	Next i
End Sub

' This function returns true if there's a collision
' Between the player and the enemy
Function collision (i as uByte)
	If enState (i) = STATESTILL Or enState (i) = STATEWALKING Then
		Return enX (i) >= plX - 2 And enX (i) <= plX + 2 And enY (i) >= plY - 2 And enY (i) <= plY + 2 
	End If
End Function

' This function changes screen
Sub changeScreen ()
	If (In (57342) bAnd 2) = 0 And plXr = 0 Then 
		nPant = nPant - 1
		plXr = SCREENWIDTH - 1
		plX = plXr * 3
		mapDraw (nPant)
	End If
		
	If (In (57342) bAnd 1) = 0 And plXr = SCREENWIDTH - 1 Then 
		nPant = nPant + 1
		plXr = 0
		plX = plXr * 3
		mapDraw (nPant)
	End If
		
	If (In (64510) bAnd 1) = 0 And plYr = 0 Then
		nPant = nPant - MAPWIDTH
		plYr = SCREENHEIGHT - 1
		plY = plYr * 3
		mapDraw (nPant)
	End If
	
	If (In (65022) bAnd 1) = 0  And plYr = SCREENHEIGHT - 1 Then
		nPant = nPant + MAPWIDTH
		plYr = 0
		plY = plYr * 3
		mapDraw (nPant)
	End If
End Sub 

' This function draws the game frame.
Sub drawFrame ()
	Dim i as uByte
	printerPutString (0, 0, 71, "abbbbbbbbbbbbbbbbbbbbbbbbbbbc")
	For i = 1 To 21
		printerPutString (0, i, 71, "d")
		printerPutString (28, i, 71, "d")
	Next i
	printerPutString (0, 22, 71, "ebc      abbbbbbbbbc     abbf")
	printerPutString (2, 23, 71, "ebbbbbbf         ebbbbbf")
	printerPutString (3, 22, 70, "jklm")
	printerPutString (7, 22, 69, "00")
	printerPutString (20, 22, 70, "nop")
	printerPutString (23, 22, 69, "00")
	printerPutString (29, 1, 65, "hh")
	printerPutString (29, 21, 65, "hh")
	For i = 3 To 19 Step 4
		printerPutString (29, i, 68, "ii")
		printerPutString (31, i, 71, "d")
	Next i
	printerPutString (28, 2, 71, "gbbc")
	printerPutString (28, 20, 71, "gbbf")
End Sub

' This function takes one life out
' and returns TRUE if no more lives are left
Function oneLifeLess ()
	Dim res as uByte = 0
	if plLives > 0 Then
		plLives = plLives - 1
	Else
		res = 1
	End If
	return res
End Function

Function normalizeNumber (n as uByte) As String
	If n < 10 Then 
		Return "0" + Str (n)
	Else
		Return Str (n)
	End If
End Function

' This sub shows current lives
Sub showLife ()
	printerPutString (23, 22, 69, normalizeNumber (plLives))
End Sub

' This sub shows current bombs
Sub showBombs ()
	printerPutString (7, 22, 69, normalizeNumber (plBombs))
End Sub

' This sub attempts to drop a bomb, and
' starts it if you can
Sub fireBomb ()
	If plBombs > 0 And bombState = STATEIDLE Then
		' Drop bomb
		bombX = plXr
		bombY = plYr
		bombTimer = 5
		bombState = STATECOUNTDOWN
		plBombs = plBombs - 1
		showBombs ()
		bombCt = 8
		makeSound (SOUNDSETBOMB)
	Else
		' Make a sound
		makeSound (SOUNDWRONG)
	End If
End Sub

' This fills the screen (the playing area) with a nifty colour
Sub fillScreen (n)
	Poke @fillWhat, n
	Asm
		ld bc, 21
		ld hl, 22561
		ld de, 6
		ld a, (fillwhat)
		push af	
		jr whitemeloop
	End Asm
fillWhat:
	Asm	
	fillwhat:
		defb 0		
	whitemeloop:
		pop af
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		inc hl
		ld  (hl), a
		add hl, de
	
		push af
		dec bc
		ld a, c
		or b
		jr nz, whitemeloop
		halt
		halt
		pop af
	End Asm
End Sub

' This sub updates the bomb and makes it explode (if active)
' It also draws it.
Function moveBomb ()
	Dim rBx as Byte
	Dim rBy as Byte
	Dim i as uByte
	Dim res as uByte
	
	rBx = 3 * bombX
	rBy = 3 * bombY
	
	res = 0
	
	If bombState = STATECOUNTDOWN Then
		subacoDrawTile (SCREENX + rBx, SCREENY + rBy, 10)
		printerPutChar (SCREENX + 1 + rBx, SCREENY + 1 + rBy, 106, 48 + bombTimer)
		
		bombCt = bombCt - 1
		If bombCt = 0 Then
			bombCt = 8
			If bombTimer > 0 Then
				bombTimer = bombTimer - 1
				Beep .01, 30
			Else
				' Explosion
				subacoDrawTile (SCREENX + rBx, SCREENY + rBy, 11)
				makeSound (SOUNDCOLLISION)
				
				If bombY > 0 Then subacoDrawTile (SCREENX + rBx, SCREENY + rBy - 3, 12): End If
				If bombY < SCREENHEIGHT - 1 Then subacoDrawTile (SCREENX + rBx, SCREENY + rBy + 3, 12): End If
				If bombX > 0 Then subacoDrawTile (SCREENX + rBx - 3, SCREENY + rBy, 12): End If
				If bombX < SCREENWIDTH - 1 Then subacoDrawTile (SCREENX + rBx + 3, SCREENY + rBy, 12): End If
				subacoDrawTile (SCREENX + rBx, SCREENY + rBy, 12)
				makeSound (SOUNDCOLLISION)
				
				If bombY > 0 And bombX > 0 Then subacoDrawTile (SCREENX + rBx - 3, SCREENY + rBy - 3, 12): End If
				If bombY < SCREENHEIGHT - 1 And bombX > 0 Then subacoDrawTile (SCREENX + rBx - 3, SCREENY + rBy + 3, 12): End If
				If bombY > 0 And bombX < SCREENWIDTH - 1 Then subacoDrawTile (SCREENX + rBx + 3, SCREENY + rBy - 3, 12): End If
				If bombY < SCREENHEIGHT - 1 And bombX < SCREENWIDTH - 1 Then subacoDrawTile (SCREENX + rBx + 3, SCREENY + rBy + 3, 12): End If
				subacoDrawTile (SCREENX + rBx, SCREENY + rBy, 0)
				makeSound (SOUNDCOLLISION)
				
				fillScreen (63)
				
				mapDrawDirect (nPant)
				Asm
					call subaco_fill_buffers
					call subaco_draw_sprites	
				End Asm
				'subacoMoveSprites ()
				
				' Check: enemies
				For i = 0 To 1
					If enState (i) <> STATEDYING And enState (i) <> STATEIDLE Then
						'If enXr (i) >= bombX - 1 And enXr (i) <= bombX + 1 And enYr (i) >= bombY - 1 And enYr (i) <= bombY + 1 Then
						If enX (i) >= rBx - 5 And enX (i) <= rBx + 5 And enY (i) >= rBy - 5 And enY (i) <= rBy + 5 Then
							enState (i) = STATEDYING
						End If
					End If 
				Next i
				
				' Check player
				If plX >= bombX - 1 And plX <= bombX + 1 And plY >= bombY - 1 And plY <= bombY + 1 Then
					res = oneLifeLess
					makeSound (SOUNDCOLLISION)
				End If
				
				' Shut down bomb
				bombState = STATEIDLE
				
				' Everyone shat on their pants
				bombScare = 1
			End If
		End If
	End If
	
	Return res
End Function

' Shows something while we setup the map and stuff
Sub splashScreen ()
	Dim message as uByte
	Dim sprs (2) as uByte
	Dim text as String
	Dim i as uByte
	Dim j as uByte
	Dim frame as uByte
	Dim hCt as uByte = 0
	
	''' Randomize Peek 23672
	''' Change when RANDOMIZE works properly.
	''' Until then:
	For i = 0 To Peek 23672
		j = Int (Rnd * 10)
	Next i
	''' Ok, it HAS been solved, but I've fond
	''' this works a treat, so it's staying :)
	
	fillScreen (0)
	sprs (0) = 0
	sprs (1) = 0
	sprs (2) = 0
	message = Int (Rnd * 10)
	
	If message = 0 Then
		text = "AROGAS ENTREGAGAS"
		sprs (1) = 26
	ElseIf message = 1 Then
		text = "LES DAREMOS UNA GOLPISA"
		sprs (0) = 24
		sprs (1) = 26
		sprs (2) = 28
	ElseIf message = 2 Then
		text = "PODEEEEEWR"
		sprs (1) = 24
	ElseIf message = 3 Then
		text = "COW SHE NO IN DA HOUSE"
		sprs (1) = 30
	ElseIf message = 4 Then
		text = "DOGS LIFE"
		sprs (0) = 138
		sprs (2) = 139
	ElseIf message = 5 Then
		text = "TORTILLA TIME"
		sprs (1) = 9
	ElseIf message = 6 Then
		text = "SE BEBES NO CANTUSICAS"
		sprs (1) = 28
	ElseIf message = 7 Then
		text = "ES MAS DE BIGOTITO"
		sprs (1) = 16
	ElseIf message = 8 Then
		text = "GET READY KIDDO"
		sprs (0) = 24
		sprs (1) = 26
		sprs (2) = 28
	Else 
		text = "BAILAN SIN CESAR"
		sprs (1) = 13
	End If
	
	printerPutString (14 - len (text) / 2, 10, 71, text)	
	
	Asm	
		push ix
		LD    HL,MUSICDATAB
		call BEEPOLA_START
		pop ix
	End asm
	
	'' Short animation
	frame = 0 
	For j = 0 To 8
		For i = 0 To 2
			If sprs (i) > 15 and sprs (i) < 32 Then
				subacoDrawTile (9 + 4 * i, 12, sprs (i) + frame)
			Else
				subacoDrawTile (9 + 4 * i, 12, sprs (i))
			End If
		Next i
		frame = 1 - frame
		For hCt = 1 To 2
			Asm
				push ix
				call BEEPOLA_NEXTNOTE
				pop ix
			End Asm
			If Inkey$ <> "" Then Exit For: End If
		Next hCt
		If Inkey$ <> "" Then Exit For: End If
	Next j
End Sub

' Title Screen
Sub title ()
	Dim frame as uByte
	Dim myInk as uByte = 1
	Dim hCt as uByte = 0
	
	Cls
	
	printerPutString (0, 0, 71, "abbbbbbbbbbbbbbbbbbbbbbbbbbbbbbc")
	printerPutString (0, 1, 71, "d                              d")
	printerPutString (0, 2, 71, "d                              d")
	printerPutString (0, 3, 71, "ebbbbbbbbbbbbbbbbbbbbbbbbbbbbbbf")
	printerPutString (11, 0, 70, "MARITRINI")
	printerPutString (4, 2, 68, "FREELANCE MONSTER SLAYER")
	printerPutString (1, 5, 7, "EN: LAS INCREIBLES VICISITUDES")
	printerPutString (2, 7, 7, "DE DESPERTARSE RESACOSA CON")
	printerPutString (2, 9, 7, "FRED EN LA CAMA Y TENER QUE")
	printerPutString (3, 11, 7, "LLEGAR MAS O MENOS PUNTUAL")
	printerPutString (8, 13, 7, "A LA PRUEBA DE ")
	printerPutString (6, 15, 69, ">MONSTRUOS VIGOROSOS")
	printerPutString (6, 17, 69, "DE PECHOS LUSTROSOS<")
	printerPutString (5, 19, 70, "FEATURING LOS FRATELLI")
	printerPutString (0, 20, 71, "bbbbbbbbbbbbbbbbbbbbbbbbbbbbbbbb")
	
	printerPutString (15, 23, 75, "THE MOJON TWINS")
	
	Asm	
		push ix
		LD    HL,MUSICDATA 
		call BEEPOLA_START
		pop ix
	End asm
	
	Do
		'printerPutString (15, 21, 64 + myInk, "PULSA UNA TECLA")
		printerPutString (15, 21, 64 + myInk, ">PRESS ANY KEY<")
		subacoDrawTile (2, 21, 16 + frame)
		subacoDrawTile (5, 21, 24 + frame)
		subacoDrawTile (8, 21, 26 + frame)
		subacoDrawTile (11, 21, 28 + frame)
		'Asm 
		'	Halt
		'End Asm
		myInk = myInk + 1: If myInk = 8 Then myInk = 1: End If
		hCt = hCt + 1
		If hCt = 2 Then
			frame = 1 - frame
			hCt = 0
		End If
		
		'' BEEPOLA!
		Asm
			push ix
			call BEEPOLA_NEXTNOTE
			pop ix
		End Asm
	Loop While Inkey = ""
	
	Cls
End Sub

' Game is ver
Sub gameOver ()
	Dim i as uByte
	fillScreen (2)
	For i = 0 To 7
		makeSound (SOUNDCOLLISION)
		makeSound (SOUNDSETBOMB)
	Next i
End Sub

' Ending sequence
Sub ending ()
	Asm
		call subaco_restore_background
	End Asm
	
	makeSound (SOUNDSETBOMB)
	Pause 52
	makeSound (SOUNDWRONG)
	Pause 40
	makeSound (SOUNDWRONG)
	Pause 20
	makeSound (SOUNDWRONG)
	Pause 65
	makeSound (SOUNDWRONG)
	Pause 15
	makeSound (SOUNDWRONG)
	Pause 10
	makeSound (SOUNDWRONG)
	Pause 50
	
	printerPutString (1, 4, 56, "        ")
	'printerPutString (1, 5, 57, " MIERDA ")
	printerPutString (1, 5, 57, " CRAP!! ")
	printerPutString (1, 6, 56, "        ")
	
	Pause 100
	
	fillScreen (9)
	'printerPutString (6, 1, 79, "POR FIN MARITRINI")
	'printerPutString (6, 3, 79, "ESTA LISTA PARA")
	'printerPutString (9, 5, 79, "LA PRUEBA!")
	
	'printerPutString (6, 9, 79, "PERO, ESPERA...")
	'printerPutString (6, 11, 79, "EL COCHE NO TIENE")
	'printerPutString (9, 13, 79, "GASOLINA!!")
	
	'printerPutString (4, 17, 79, "CREO QUE SU CARRERA")
	'printerPutString (5, 19, 79, "ACABA DE TERMINAR.")
	
	printerPutString (10, 1, 79, "AT LAST,")
	printerPutString (5, 3, 79, "MARITRINI IS READY")
	printerPutString (5, 5, 79, "FOR THE AUDITION!!")
	printerPutString (6, 9, 79, "BUT... OH WAIT.")
	printerPutString (6, 11, 79, "THE CAR HAS RAN")
	printerPutString (8, 13, 79, "OUT OF GAS!!")
	printerPutString (5, 17, 79, "I THINK HER CAREER")
	printerPutString (4, 19, 79, "HAS COME TO AN END.")
	
	For i = 0 To 7
		makeSound (SOUNDCOLLISION)
		makeSound (SOUNDSETBOMB)
	Next i
	
	While Inkey$ <> "": Wend
	While Inkey$ = "": Wend
	
End Sub